@@ -163,6 +163,8 @@ struct _GreeterBackgroundPrivate
 
     GList* active_monitors_config;
     const Monitor* active_monitor;
+    gboolean active_monitor_change_in_progress;
+    gint64 active_monitor_change_last_timestamp;
 
     /* List of monitors <Monitor*> with user-background=true*/
     GSList* customized_monitors;
@@ -379,6 +381,8 @@ greeter_background_set_active_monitor_config(GreeterBackground* background,
 
     g_list_free_full(priv->active_monitors_config, g_free);
     priv->active_monitors_config = NULL;
+    priv->active_monitor_change_in_progress = FALSE;
+    priv->active_monitor_change_last_timestamp = 0;
 
     priv->follow_cursor = FALSE;
     priv->follow_cursor_to_init = FALSE;
@@ -726,11 +730,34 @@ greeter_background_set_active_monitor(GreeterBackground* background,
 {
     GreeterBackgroundPrivate* priv = background->priv;
     gint x, y;
+    gint64 timestamp;
+
+    if (priv->active_monitor_change_in_progress)
+        return;
+
+    /* Prevents infinite signal emmission between two monitors (LP: #1410406, #1509780)
+     * There are some rare scenarios when using multiple monitors that cause the greeter
+     * to switch back and forth between the monitors indefinitely. By comparing the
+     * timestamp at this precision (1/10th of a second), this should no longer be
+     * possible.
+     */
+    if (active != NULL)
+    {
+        timestamp = floor(g_get_monotonic_time () * 0.00001);
+        if (timestamp == priv->active_monitor_change_last_timestamp)
+        {
+            g_debug("[Background] Preventing infinite monitor loop");
+            return;
+        }
+    }
+
+    priv->active_monitor_change_in_progress = TRUE;
+    priv->active_monitor_change_last_timestamp = floor(g_get_monotonic_time () * 0.00001);
 
     if(active && !active->background)
     {
         if(priv->active_monitor)
-            return;
+            goto active_monitor_change_complete;
         active = NULL;
     }
 
@@ -754,7 +781,9 @@ greeter_background_set_active_monitor(GreeterBackground* background,
         if(!active)
         {
             gint num = greeter_screen_get_primary_monitor(priv->screen);
-            g_return_if_fail((guint)num < priv->monitors_size);
+            if ((guint)num >= priv->monitors_size)
+                goto active_monitor_change_complete;
+
             active = &priv->monitors[num];
             if(!active->background || !greeter_background_monitor_enabled(background, active))
                 active = NULL;
@@ -797,15 +826,16 @@ greeter_background_set_active_monitor(GreeterBackground* background,
                       priv->active_monitor->name, priv->active_monitor->number);
         else
             g_warning("[Background] Active monitor is not specified, failed to identify. Active monitor stays the same: <not defined>");
-        return;
+        goto active_monitor_change_complete;
     }
 
     if(active == priv->active_monitor)
-        return;
+        goto active_monitor_change_complete;
 
     priv->active_monitor = active;
 
-    g_return_if_fail(priv->active_monitor != NULL);
+    if (priv->active_monitor == NULL)
+        goto active_monitor_change_complete;
 
     if(priv->child)
     {
@@ -834,6 +864,10 @@ greeter_background_set_active_monitor(GreeterBackground* background,
         greeter_background_set_cursor_position(background,
                                                active->geometry.x + active->geometry.width/2,
                                                active->geometry.y + active->geometry.height/2);
+
+active_monitor_change_complete:
+    priv->active_monitor_change_in_progress = FALSE;
+    return;
 }
 
 static void
@@ -1582,7 +1616,7 @@ create_root_surface(GdkScreen* screen)
     height = greeter_screen_get_height (screen);
 
     /* Open a new connection so with Retain Permanent so the pixmap remains when the greeter quits */
-    gdk_flush ();
+    greeter_gdk_flush ();
     display = XOpenDisplay (gdk_display_get_name (gdk_screen_get_display (screen)));
     if (!display)
     {
@@ -1639,7 +1673,7 @@ set_root_pixmap_id(GdkScreen* screen,
                 XFree (data_root);
                 XFree (data_esetroot);
 
-                gdk_error_trap_push ();
+                greeter_error_trap_push ();
                 if (xrootpmap && xrootpmap == esetrootpmap) {
                     XKillClient (display, xrootpmap);
                 }
@@ -1648,7 +1682,7 @@ set_root_pixmap_id(GdkScreen* screen,
                 }
 
                 XSync (display, False);
-                gdk_error_trap_pop_ignored ();
+                greeter_error_trap_pop_ignored ();
             }
         }
     }
